//------------------------------------------------------------------------------
// File: Compressor.h
//------------------------------------------------------------------------------

#ifndef __COMPRESSOR__
#define __COMPRESSOR__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define COMPRESSOR_TARGET 25000		/* Target level */

#define COMPRESSOR_GAINMAX_LIGHT		5	/* The maximum amount to amplify by */
#define COMPRESSOR_GAINSMOOTH_LIGHT		8	/* How much inertia ramping has*/
#define COMPRESSOR_BUCKETS_TIME_LIGHT	3	/* Length of history in seconds */

#define COMPRESSOR_GAINMAX_NORMAL		5	/* The maximum amount to amplify by */
#define COMPRESSOR_GAINSMOOTH_NORMAL	2	/* How much inertia ramping has*/
#define COMPRESSOR_BUCKETS_TIME_NORMAL	1	/* Length of history in seconds */

#define COMPRESSOR_GAINMAX_STRONG		6	/* The maximum amount to amplify by */
#define COMPRESSOR_GAINSMOOTH_STRONG	1	/* How much inertia ramping has*/
#define COMPRESSOR_BUCKETS_TIME_STRONG	1	/* Length of history in seconds */

#define COMPRESSOR_GAINSHIFT 10		/* How fine-grained the gain is */

class Compressor
{
	bool m_anticlip;
	int m_target;
	int m_gainmax;
	int m_gainsmooth;
	int m_bucketsTime;
	int m_currentBucketCount;
	int m_currentBucketPeak;
	int m_samplesPerBucket;
	int *m_peaks ;
	int m_gainCurrent, m_gainTarget;
	int m_pn;
	int m_clip;
	
public:
	Compressor(int samplesPerSec, bool anticlip, int target, int gainmax, int gainsmooth, int bucketsTime);
	~Compressor();

	Configure(int samplesPerSec, bool anticlip, int target, int gainmax, int gainsmooth, int bucketsTime);
	void ProcessFlow16(short *input, short *output, int sampleCount);
	double GetCurrentGain() { return (double)m_gainCurrent / (double)(1 << COMPRESSOR_GAINSHIFT); }
	int GetClipCount() { return m_clip; }
};

#endif // __COMPRESSOR__
